/*-
 * Copyright (C) @BABOLO  2005 Mar 30
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#ifndef lint
static const char copyright[] = "\
@(#)Copyright (C) @BABOLO  2005 Mar 30\n\
@(#)All rights reserved.\n";
static const char rcsid[] = "$Id: int_set.c,v 1.1 2005/03/30 01:42:29 babolo Exp $";
#endif /* not lint */

#include <sys/param.h>
#include <sys/types.h>
#include <postgres.h>
#include <funcapi.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <fmgr.h>
#include <nodes/nodes.h>
#include <nodes/execnodes.h>
#define fctx ((results*)funcctx->user_fctx)

typedef struct {
    long cur;
    long max;
    long stp;
} results;

PG_FUNCTION_INFO_V1(int4_set);
Datum
int4_set(PG_FUNCTION_ARGS) {
    int cur;
    FuncCallContext  *funcctx;
    MemoryContext oldcontext;

    if  (SRF_IS_FIRSTCALL()) {
        funcctx = SRF_FIRSTCALL_INIT();
        oldcontext = MemoryContextSwitchTo(funcctx->multi_call_memory_ctx);
        if  (PG_ARGISNULL(0) || PG_ARGISNULL(1) || PG_ARGISNULL(2)) { SRF_RETURN_DONE(funcctx); }
        fctx = (results*)palloc(sizeof(results));
        if  (fctx == NULL)
                     ereport(ERROR, (errcode(ERRCODE_OUT_OF_MEMORY), errmsg("ip_set: out of memory")));
        MemSet(fctx, 0, sizeof(results));
        fctx->cur = PG_GETARG_INT32(0);
        fctx->max = PG_GETARG_INT32(1);
        fctx->stp = PG_GETARG_INT32(2);
        if (fctx->stp > 0) {
            if  (fctx->max < fctx->cur) { SRF_RETURN_DONE(funcctx); }
        }else if (fctx->stp < 0) {
            if (fctx->max > fctx->cur) { SRF_RETURN_DONE(funcctx); }
        }else{
            ereport(ERROR, ( errcode(ERRCODE_INVALID_PARAMETER_VALUE)
                           , errmsg("int4_set: Out of range")
                           , errdetail( "Parameter $3 = %ld can't be 0", fctx->stp)
                   )       );
        }
        MemoryContextSwitchTo(oldcontext);
    }
    funcctx = SRF_PERCALL_SETUP();
    if  ((fctx->max != fctx->cur) && ((fctx->max < fctx->cur) == (fctx->stp > 0))) {
        SRF_RETURN_DONE(funcctx);
    }else{
        cur = fctx->cur;
        fctx->cur += fctx->stp;
        SRF_RETURN_NEXT(funcctx, Int32GetDatum(cur));
}   }

PG_FUNCTION_INFO_V1(int2_set);
Datum
int2_set(PG_FUNCTION_ARGS) {
    int cur;
    FuncCallContext  *funcctx;
    MemoryContext oldcontext;

    if  (SRF_IS_FIRSTCALL()) {
        funcctx = SRF_FIRSTCALL_INIT();
        oldcontext = MemoryContextSwitchTo(funcctx->multi_call_memory_ctx);
        if  (PG_ARGISNULL(0) || PG_ARGISNULL(1) || PG_ARGISNULL(2)) { SRF_RETURN_DONE(funcctx); }
        fctx = (results*)palloc(sizeof(results));
        if  (fctx == NULL)
                     ereport(ERROR, (errcode(ERRCODE_OUT_OF_MEMORY), errmsg("ip_set: out of memory")));
        MemSet(fctx, 0, sizeof(results));
        fctx->cur = PG_GETARG_INT16(0);
        fctx->max = PG_GETARG_INT16(1);
        fctx->stp = PG_GETARG_INT16(2);
        if (fctx->stp > 0) {
            if  (fctx->max < fctx->cur) { SRF_RETURN_DONE(funcctx); }
        }else if (fctx->stp < 0) {
            if (fctx->max > fctx->cur) { SRF_RETURN_DONE(funcctx); }
        }else{
            ereport(ERROR, ( errcode(ERRCODE_INVALID_PARAMETER_VALUE)
                           , errmsg("int2_set: Out of range")
                           , errdetail( "Parameter $3 = %ld can't be 0", fctx->stp)
                   )       );
        }
        MemoryContextSwitchTo(oldcontext);
    }
    funcctx = SRF_PERCALL_SETUP();
    if  ((fctx->max != fctx->cur) && ((fctx->max < fctx->cur) == (fctx->stp > 0))) {
        SRF_RETURN_DONE(funcctx);
    }else{
        cur = fctx->cur;
        fctx->cur += fctx->stp;
        SRF_RETURN_NEXT(funcctx, Int16GetDatum(cur));
}   }
